#include <PalmOS.h>
#include "res.h"
#include "persist.h"
#include "prefsmodal.h"

static void MainFormInit(FormType *form){
    struct myPrefs prefs;
    // ensure default prefs are set
    MyPrefsGet(&prefs);
    MyPrefsSet(&prefs);
}

static void MainFormDeInit(FormType *form){
}

#pragma pack(push,1)
struct chapter {
    UInt16 day;
    UInt8 level1, level2, level3;
};
#pragma pack(pop)

static void MainFormUpdateWithData(FormType *form, UInt16 day, UInt16 startDay){
    MemHandle paragraphsH = DmGetResource('tSTR', ParagraphData);
    UInt16 *paragraphs;
    MemHandle chaptersH = DmGetResource('tSTR', ChapterData);
    struct chapter *chapters, *chapter = NULL;
    DateType date;
    UInt16 idx, endPara, n;
    ControlType *control;
    char *buf;

#define _GETLABEL(controlID) \
    idx = FrmGetObjectIndex(form, controlID); \
    control = FrmGetObjectPtr(form, idx); \
    buf = (char *)CtlGetLabel(control);

    _GETLABEL(DayValue)
    FrmHideObject(form, idx);
    StrPrintF(buf, "%d", 1+day);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    _GETLABEL(DateValue)
    FrmHideObject(form, idx);
    date.year = 1;
    date.month = 1;
    date.day = 1;
    DateAdjust(&date, (startDay + day) % 365);
    DateToAscii(date.month, date.day, date.year, dfMDYLongWithComma, buf);
    for(char *c = buf + 5; c < buf + 7; ++c){
        if(*c == ','){
            *c = 0;
            break;
        }
    }
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    paragraphs = MemHandleLock(paragraphsH);
    if(day == 364){
        endPara = 2865;
    } else{
        endPara = paragraphs[day+1];
    }
    n = endPara - paragraphs[day];
    _GETLABEL(ParaNone)
    FrmHideObject(form, idx);
    _GETLABEL(ParaLabel)
    FrmHideObject(form, idx);
    _GETLABEL(ParaValue)
    FrmHideObject(form, idx);
    if(n == 0){
        _GETLABEL(ParaNone)
        FrmShowObject(form, idx);
    } else{
        _GETLABEL(ParaLabel)
        FrmShowObject(form, idx);
        _GETLABEL(ParaValue)
        StrPrintF(buf, "%d-%d.", paragraphs[day], endPara-1);
        CtlSetLabel(control, buf);
        FrmShowObject(form, idx);
    }
    MemHandleUnlock(paragraphsH);

    chapters = MemHandleLock(chaptersH);
    for(UInt8 i = 25; i > 0; --i){
        if(chapters[i].day <= day){
            chapter = chapters + i;
            break;
        }
    }
    if(!chapter) chapter = chapters;
    _GETLABEL(Chapter1)
    FrmHideObject(form, idx);
    SysStringByIndex(ChapterStrings, chapter->level1, buf, 18);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    _GETLABEL(Chapter2)
    FrmHideObject(form, idx);
    if(chapter->level2 != chapter->level1){
        SysStringByIndex(ChapterStrings, chapter->level2, buf, 18);
        CtlSetLabel(control, buf);
        FrmShowObject(form, idx);
    }
    _GETLABEL(Chapter3)
    FrmHideObject(form, idx);
    if(chapter->level3 != chapter->level2){
        SysStringByIndex(ChapterStrings, chapter->level3, buf, 18);
        CtlSetLabel(control, buf);
        FrmShowObject(form, idx);
    }
    MemHandleUnlock(chaptersH);
    
#undef _GETLABEL
}
static void MainFormUpdate(FormType *form){
    struct myPrefs prefs;
    MyPrefsGet(&prefs);
    MainFormUpdateWithData(form, prefs.day, prefs.startDay);
}

static void _Increment(FormType *form, Int8 d){
    struct myPrefs prefs;
    MyPrefsGet(&prefs);
    prefs.day = (prefs.day + 365 + d) % 365;
    MainFormUpdateWithData(form, prefs.day, prefs.startDay);
    MyPrefsSet(&prefs);
}

Boolean MainFormHandleEvent(EventType *event){
    FormType *form, *dialog;
    UInt16 choice;
    switch(event->eType){
    case frmOpenEvent:
        form = FrmGetActiveForm();
        MainFormInit(form);
        MainFormUpdate(form);
        FrmDrawForm(form);
        return true;
    case frmUpdateEvent:
        form = FrmGetActiveForm();
        FrmDrawForm(form);
        return true;
    case frmCloseEvent:
        form = FrmGetActiveForm();
        MainFormDeInit(form);
        return false;
    case ctlSelectEvent:
        switch(event->data.ctlSelect.controlID){
        case PrefsBtn:
            form = FrmGetActiveForm();
            dialog = FrmInitForm(PrefsModal);
            PrefsModalInit(dialog);
            FrmSetEventHandler(dialog, PrefsModalHandleEvent);
            choice = FrmDoDialog(dialog);
            if(choice == OKBtn){
                PrefsModalSave(dialog);
                MainFormUpdate(form);
            }
            FrmDeleteForm(dialog);
            return true;
        case AboutBtn:
            dialog = FrmInitForm(AboutDialog);
            FrmDoDialog(dialog);
            FrmDeleteForm(dialog);
            return true;
        }
        return false;
    case ctlRepeatEvent:
        form = FrmGetActiveForm();
        switch(event->data.ctlRepeat.controlID){
        case UpArrow:
            _Increment(form, 1);
            break;
        case DownArrow:
            _Increment(form, -1);
            break;
        }
        return false;
    case keyDownEvent:
        form = FrmGetActiveForm();
        switch(event->data.keyDown.chr){
        case pageUpChr:
            _Increment(form, 1);
            break;
        case pageDownChr:
            _Increment(form, -1);
            break;
        default: break;
        }
        return true;
    default:
        return false;
    }
}
